pub struct LoopInfo {
pub name: Option<Symbol>,
pub break_label: Label,
pub continue_label: Label,
pub loop_result: Destination,
}Expand description
Information about a loop.
Fields
name: Option<Symbol>The name of the loop, if specified.
break_label: LabelThe label for the break operation of the loop.
continue_label: LabelThe label for the continue operation of the loop.
loop_result: DestinationThe desination to store the loops result into.